@@ -1,21 +1,21 @@
 # This file is distributed under the same license as the Django package.
 #
 # The *_FORMAT strings use the Django date format syntax,
-# see http://docs.djangoproject.com/en/dev/ref/templates/builtins/#date
-DATE_FORMAT = 'd F Y'
-TIME_FORMAT = 'H:i'
+# see https://docs.djangoproject.com/en/dev/ref/templates/builtins/#date
+DATE_FORMAT = "d F Y"
+TIME_FORMAT = "H:i"
 # DATETIME_FORMAT =
 # YEAR_MONTH_FORMAT =
-MONTH_DAY_FORMAT = 'j F'
-SHORT_DATE_FORMAT = 'd.m.Y'
+MONTH_DAY_FORMAT = "j F"
+SHORT_DATE_FORMAT = "d.m.Y"
 # SHORT_DATETIME_FORMAT =
 # FIRST_DAY_OF_WEEK =
 
 # The *_INPUT_FORMATS strings use the Python strftime format syntax,
-# see http://docs.python.org/library/datetime.html#strftime-strptime-behavior
+# see https://docs.python.org/library/datetime.html#strftime-strptime-behavior
 # DATE_INPUT_FORMATS =
 # TIME_INPUT_FORMATS =
 # DATETIME_INPUT_FORMATS =
-DECIMAL_SEPARATOR = ','
-THOUSAND_SEPARATOR = ' '  # Non-breaking space
+DECIMAL_SEPARATOR = ","
+THOUSAND_SEPARATOR = " "  # Non-breaking space
 # NUMBER_GROUPING =
