@@ -4,11 +4,9 @@ import os
 from django.apps import apps
 from django.conf import settings
 from django.contrib.staticfiles import utils
-from django.core.checks import Error
+from django.core.checks import Error, Warning
 from django.core.exceptions import ImproperlyConfigured
-from django.core.files.storage import (
-    FileSystemStorage, Storage, default_storage,
-)
+from django.core.files.storage import FileSystemStorage, Storage, default_storage
 from django.utils._os import safe_join
 from django.utils.functional import LazyObject, empty
 from django.utils.module_loading import import_string
@@ -21,10 +19,11 @@ class BaseFinder:
     """
     A base file finder to be used for custom staticfiles finder classes.
     """
+
     def check(self, **kwargs):
         raise NotImplementedError(
-            'subclasses may provide a check() method to verify the finder is '
-            'configured correctly.'
+            "subclasses may provide a check() method to verify the finder is "
+            "configured correctly."
         )
 
     def find(self, path, all=False):
@@ -34,14 +33,18 @@ class BaseFinder:
         If the ``all`` parameter is False (default) return only the first found
         file path; if True, return a list of all found files paths.
         """
-        raise NotImplementedError('subclasses of BaseFinder must provide a find() method')
+        raise NotImplementedError(
+            "subclasses of BaseFinder must provide a find() method"
+        )
 
     def list(self, ignore_patterns):
         """
         Given an optional list of paths to ignore, return a two item iterable
         consisting of the relative path and storage instance.
         """
-        raise NotImplementedError('subclasses of BaseFinder must provide a list() method')
+        raise NotImplementedError(
+            "subclasses of BaseFinder must provide a list() method"
+        )
 
 
 class FileSystemFinder(BaseFinder):
@@ -49,6 +52,7 @@ class FileSystemFinder(BaseFinder):
     A static files finder that uses the ``STATICFILES_DIRS`` setting
     to locate files.
     """
+
     def __init__(self, app_names=None, *args, **kwargs):
         # List of locations with static files
         self.locations = []
@@ -58,7 +62,7 @@ class FileSystemFinder(BaseFinder):
             if isinstance(root, (list, tuple)):
                 prefix, root = root
             else:
-                prefix = ''
+                prefix = ""
             if (prefix, root) not in self.locations:
                 self.locations.append((prefix, root))
         for prefix, root in self.locations:
@@ -70,26 +74,43 @@ class FileSystemFinder(BaseFinder):
     def check(self, **kwargs):
         errors = []
         if not isinstance(settings.STATICFILES_DIRS, (list, tuple)):
-            errors.append(Error(
-                'The STATICFILES_DIRS setting is not a tuple or list.',
-                hint='Perhaps you forgot a trailing comma?',
-                id='staticfiles.E001',
-            ))
+            errors.append(
+                Error(
+                    "The STATICFILES_DIRS setting is not a tuple or list.",
+                    hint="Perhaps you forgot a trailing comma?",
+                    id="staticfiles.E001",
+                )
+            )
+            return errors
         for root in settings.STATICFILES_DIRS:
             if isinstance(root, (list, tuple)):
                 prefix, root = root
-                if prefix.endswith('/'):
-                    errors.append(Error(
-                        'The prefix %r in the STATICFILES_DIRS setting must '
-                        'not end with a slash.' % prefix,
-                        id='staticfiles.E003',
-                    ))
-            if settings.STATIC_ROOT and os.path.abspath(settings.STATIC_ROOT) == os.path.abspath(root):
-                errors.append(Error(
-                    'The STATICFILES_DIRS setting should not contain the '
-                    'STATIC_ROOT setting.',
-                    id='staticfiles.E002',
-                ))
+                if prefix.endswith("/"):
+                    errors.append(
+                        Error(
+                            "The prefix %r in the STATICFILES_DIRS setting must "
+                            "not end with a slash." % prefix,
+                            id="staticfiles.E003",
+                        )
+                    )
+            if settings.STATIC_ROOT and os.path.abspath(
+                settings.STATIC_ROOT
+            ) == os.path.abspath(root):
+                errors.append(
+                    Error(
+                        "The STATICFILES_DIRS setting should not contain the "
+                        "STATIC_ROOT setting.",
+                        id="staticfiles.E002",
+                    )
+                )
+            if not os.path.isdir(root):
+                errors.append(
+                    Warning(
+                        f"The directory '{root}' in the STATICFILES_DIRS setting "
+                        f"does not exist.",
+                        id="staticfiles.W004",
+                    )
+                )
         return errors
 
     def find(self, path, all=False):
@@ -113,10 +134,10 @@ class FileSystemFinder(BaseFinder):
         absolute path (or ``None`` if no match).
         """
         if prefix:
-            prefix = '%s%s' % (prefix, os.sep)
+            prefix = "%s%s" % (prefix, os.sep)
             if not path.startswith(prefix):
                 return None
-            path = path[len(prefix):]
+            path = path[len(prefix) :]
         path = safe_join(root, path)
         if os.path.exists(path):
             return path
@@ -126,9 +147,11 @@ class FileSystemFinder(BaseFinder):
         List all files in all locations.
         """
         for prefix, root in self.locations:
-            storage = self.storages[root]
-            for path in utils.get_files(storage, ignore_patterns):
-                yield path, storage
+            # Skip nonexistent directories.
+            if os.path.isdir(root):
+                storage = self.storages[root]
+                for path in utils.get_files(storage, ignore_patterns):
+                    yield path, storage
 
 
 class AppDirectoriesFinder(BaseFinder):
@@ -136,8 +159,9 @@ class AppDirectoriesFinder(BaseFinder):
     A static files finder that looks in the directory of each app as
     specified in the source_dir attribute.
     """
+
     storage_class = FileSystemStorage
-    source_dir = 'static'
+    source_dir = "static"
 
     def __init__(self, app_names=None, *args, **kwargs):
         # The list of apps that are handled
@@ -150,7 +174,8 @@ class AppDirectoriesFinder(BaseFinder):
             app_configs = [ac for ac in app_configs if ac.name in app_names]
         for app_config in app_configs:
             app_storage = self.storage_class(
-                os.path.join(app_config.path, self.source_dir))
+                os.path.join(app_config.path, self.source_dir)
+            )
             if os.path.isdir(app_storage.location):
                 self.storages[app_config.name] = app_storage
                 if app_config.name not in self.apps:
@@ -162,7 +187,7 @@ class AppDirectoriesFinder(BaseFinder):
         List all files in all app storages.
         """
         for storage in self.storages.values():
-            if storage.exists(''):  # check if storage location exists
+            if storage.exists(""):  # check if storage location exists
                 for path in utils.get_files(storage, ignore_patterns):
                     yield path, storage
 
@@ -187,12 +212,11 @@ class AppDirectoriesFinder(BaseFinder):
         Find a requested static file in an app's static locations.
         """
         storage = self.storages.get(app)
-        if storage:
-            # only try to find a file if the source dir actually exists
-            if storage.exists(path):
-                matched_path = storage.path(path)
-                if matched_path:
-                    return matched_path
+        # Only try to find a file if the source dir actually exists.
+        if storage and storage.exists(path):
+            matched_path = storage.path(path)
+            if matched_path:
+                return matched_path
 
 
 class BaseStorageFinder(BaseFinder):
@@ -200,15 +224,18 @@ class BaseStorageFinder(BaseFinder):
     A base static files finder to be used to extended
     with an own storage class.
     """
+
     storage = None
 
     def __init__(self, storage=None, *args, **kwargs):
         if storage is not None:
             self.storage = storage
         if self.storage is None:
-            raise ImproperlyConfigured("The staticfiles storage finder %r "
-                                       "doesn't have a storage class "
-                                       "assigned." % self.__class__)
+            raise ImproperlyConfigured(
+                "The staticfiles storage finder %r "
+                "doesn't have a storage class "
+                "assigned." % self.__class__
+            )
         # Make sure we have a storage instance here.
         if not isinstance(self.storage, (Storage, LazyObject)):
             self.storage = self.storage()
@@ -219,7 +246,7 @@ class BaseStorageFinder(BaseFinder):
         Look for files in the default file storage, if it's local.
         """
         try:
-            self.storage.path('')
+            self.storage.path("")
         except NotImplementedError:
             pass
         else:
@@ -244,15 +271,18 @@ class DefaultStorageFinder(BaseStorageFinder):
     """
     A static files finder that uses the default storage backend.
     """
+
     storage = default_storage
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        base_location = getattr(self.storage, 'base_location', empty)
+        base_location = getattr(self.storage, "base_location", empty)
         if not base_location:
-            raise ImproperlyConfigured("The storage backend of the "
-                                       "staticfiles finder %r doesn't have "
-                                       "a valid location." % self.__class__)
+            raise ImproperlyConfigured(
+                "The storage backend of the "
+                "staticfiles finder %r doesn't have "
+                "a valid location." % self.__class__
+            )
 
 
 def find(path, all=False):
@@ -290,6 +320,7 @@ def get_finder(import_path):
     """
     Finder = import_string(import_path)
     if not issubclass(Finder, BaseFinder):
-        raise ImproperlyConfigured('Finder "%s" is not a subclass of "%s"' %
-                                   (Finder, BaseFinder))
+        raise ImproperlyConfigured(
+            'Finder "%s" is not a subclass of "%s"' % (Finder, BaseFinder)
+        )
     return Finder()
