@@ -1,12 +1,13 @@
 import operator
 
-from django.db import InterfaceError
+from django.db import DataError, InterfaceError
 from django.db.backends.base.features import BaseDatabaseFeatures
+from django.db.backends.postgresql.psycopg_any import is_psycopg3
 from django.utils.functional import cached_property
 
 
 class DatabaseFeatures(BaseDatabaseFeatures):
-    minimum_database_version = (11,)
+    minimum_database_version = (12,)
     allows_group_by_selected_pks = True
     can_return_columns_from_insert = True
     can_return_rows_from_bulk_insert = True
@@ -21,11 +22,13 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     has_select_for_update_skip_locked = True
     has_select_for_no_key_update = True
     can_release_savepoints = True
+    supports_comments = True
     supports_tablespaces = True
     supports_transactions = True
     can_introspect_materialized_views = True
     can_distinct_on_fields = True
     can_rollback_ddl = True
+    schema_editor_uses_clientside_param_binding = True
     supports_combined_alters = True
     nulls_order_largest = True
     closed_cursor_error_class = InterfaceError
@@ -49,6 +52,13 @@ class DatabaseFeatures(BaseDatabaseFeatures):
             V_I := P_I;
         END;
     $$ LANGUAGE plpgsql;"""
+    create_test_table_with_composite_primary_key = """
+        CREATE TABLE test_table_composite_pk (
+            column_1 INTEGER NOT NULL,
+            column_2 INTEGER NOT NULL,
+            PRIMARY KEY(column_1, column_2)
+        )
+    """
     requires_casted_case_in_updates = True
     supports_over_clause = True
     only_supports_unbounded_with_preceding_and_following = True
@@ -72,8 +82,38 @@ class DatabaseFeatures(BaseDatabaseFeatures):
             "indexes.tests.SchemaIndexesNotPostgreSQLTests."
             "test_create_index_ignores_opclasses",
         },
+        "PostgreSQL requires casting to text.": {
+            "lookup.tests.LookupTests.test_textfield_exact_null",
+        },
     }
 
+    @cached_property
+    def django_test_expected_failures(self):
+        expected_failures = set()
+        if self.uses_server_side_binding:
+            expected_failures.update(
+                {
+                    # Parameters passed to expressions in SELECT and GROUP BY
+                    # clauses are not recognized as the same values when using
+                    # server-side binding cursors (#34255).
+                    "aggregation.tests.AggregateTestCase."
+                    "test_group_by_nested_expression_with_params",
+                }
+            )
+        return expected_failures
+
+    @cached_property
+    def uses_server_side_binding(self):
+        options = self.connection.settings_dict["OPTIONS"]
+        return is_psycopg3 and options.get("server_side_binding") is True
+
+    @cached_property
+    def prohibits_null_characters_in_text_exception(self):
+        if is_psycopg3:
+            return DataError, "PostgreSQL text fields cannot contain NUL (0x00) bytes"
+        else:
+            return ValueError, "A string literal cannot contain NUL (0x00) characters."
+
     @cached_property
     def introspected_field_types(self):
         return {
@@ -83,10 +123,6 @@ class DatabaseFeatures(BaseDatabaseFeatures):
             "PositiveSmallIntegerField": "SmallIntegerField",
         }
 
-    @cached_property
-    def is_postgresql_12(self):
-        return self.connection.pg_version >= 120000
-
     @cached_property
     def is_postgresql_13(self):
         return self.connection.pg_version >= 130000
@@ -96,8 +132,5 @@ class DatabaseFeatures(BaseDatabaseFeatures):
         return self.connection.pg_version >= 140000
 
     has_bit_xor = property(operator.attrgetter("is_postgresql_14"))
-    supports_covering_gist_indexes = property(operator.attrgetter("is_postgresql_12"))
     supports_covering_spgist_indexes = property(operator.attrgetter("is_postgresql_14"))
-    supports_non_deterministic_collations = property(
-        operator.attrgetter("is_postgresql_12")
-    )
+    supports_unlimited_charfield = True
