@@ -659,6 +659,8 @@ class FilterExpression:
         <Variable: 'variable'>
     """
 
+    __slots__ = ("token", "filters", "var", "is_var")
+
     def __init__(self, token, parser):
         self.token = token
         matches = filter_re.finditer(token)
@@ -705,9 +707,10 @@ class FilterExpression:
 
         self.filters = filters
         self.var = var_obj
+        self.is_var = isinstance(var_obj, Variable)
 
     def resolve(self, context, ignore_failures=False):
-        if isinstance(self.var, Variable):
+        if self.is_var:
             try:
                 obj = self.var.resolve(context)
             except VariableDoesNotExist:
@@ -789,6 +792,8 @@ class Variable:
     (The example assumes VARIABLE_ATTRIBUTE_SEPARATOR is '.')
     """
 
+    __slots__ = ("var", "literal", "lookups", "translate", "message_context")
+
     def __init__(self, var):
         self.var = var
         self.literal = None
@@ -908,15 +913,18 @@ class Variable:
                         try:  # method call (assuming no args required)
                             current = current()
                         except TypeError:
-                            signature = inspect.signature(current)
                             try:
-                                signature.bind()
-                            except TypeError:  # arguments *were* required
-                                current = (
-                                    context.template.engine.string_if_invalid
-                                )  # invalid method call
+                                signature = inspect.signature(current)
+                            except ValueError:  # No signature found.
+                                current = context.template.engine.string_if_invalid
                             else:
-                                raise
+                                try:
+                                    signature.bind()
+                                except TypeError:  # Arguments *were* required.
+                                    # Invalid method call.
+                                    current = context.template.engine.string_if_invalid
+                                else:
+                                    raise
         except Exception as e:
             template_name = getattr(context, "template_name", None) or "unknown"
             logger.debug(
@@ -973,9 +981,6 @@ class Node:
                     )
             raise
 
-    def __iter__(self):
-        yield self
-
     def get_nodes_by_type(self, nodetype):
         """
         Return a list of all nodes (within this node and its nodelist)
